<?php

namespace Nwb\CyberSecModule;

use Nwb\CyberSecModule\Config\SecurityPatterns;
use Nwb\CyberSecModule\Exceptions\CyberSecConfigurationException;
use Nwb\CyberSecModule\Exceptions\CyberSecFileException;
use Nwb\CyberSecModule\Exceptions\CyberSecNetworkException;
use Exception;

/**
 * Module de remontée de piratage
 *
 * Ce module détecte les requêtes malveillantes et les remonte
 * automatiquement à un endpoint centralisé.
 */
class CyberSecModule
{
    private $config;
    private $queueFile;
    private $patterns;

    /**
     * Configuration par défaut
     */
    private const DEFAULT_CONFIG = [
        'endpoint' => 'https://n8n.kub.nwb.fr/webhook/signalHackingAttempt',
        'timeout' => 2,
        'max_retries' => 3,
        'queue_file' => '/tmp/cybersec_queue.json',
        'compression' => true,
        'patterns_file' => null, // Fichier externe YAML/JSON pour les patterns
        'post_size_threshold' => 1048576, // 1MB par défaut pour les données POST
        'headers_detection_enabled' => true // Détection des en-têtes activable/désactivable
    ];

    /**
     * Clés sensibles à hacher
     */
    private const SENSITIVE_KEYS = [
        'HTTP_AUTHORIZATION',
        'HTTP_COOKIE',
        'PHP_AUTH_PW',
        'PHP_AUTH_USER'
    ];

    public function __construct(array $config = [])
    {
        $this->config = $this->buildConfig($config);
        $this->queueFile = $this->config['queue_file'];
        $this->patterns = $this->loadPatterns();
    }

    /**
     * Ajoute des patterns personnalisés à une catégorie existante
     *
     * @param string $category Catégorie (uri, post, headers)
     * @param array $patterns Nouveaux patterns à ajouter
     * @return void
     */
    public function addCustomPatterns(string $category, array $patterns): void
    {
        if (!isset($this->patterns[$category])) {
            throw new \InvalidArgumentException(
                "Catégorie invalide: $category. Catégories disponibles: " . implode(', ', array_keys($this->patterns))
            );
        }

        $this->patterns[$category] = array_merge($this->patterns[$category], $patterns);
    }

    /**
     * Récupère les patterns actuellement chargés
     *
     * @param string|null $category Catégorie spécifique ou null pour toutes
     * @return array
     */
    public function getPatterns(?string $category = null): array
    {
        if ($category === null) {
            return $this->patterns;
        }

        if (!isset($this->patterns[$category])) {
            throw new \InvalidArgumentException("Catégorie invalide: $category");
        }

        return $this->patterns[$category];
    }

    /**
     * Signale une tentative de piratage
     *
     * @param string $pattern Description du motif détecté
     * @param array $additionalData Données supplémentaires optionnelles
     * @return bool True si le signal a été envoyé avec succès
     */
    public function signalHackingAttempt(string $pattern, array $additionalData = []): bool
    {
        $payload = $this->buildPayload($pattern, $additionalData);

        if ($this->sendPayload($payload)) {
            return true;
        }

        // En cas d'échec, ajouter à la file d'attente
        $this->addToQueue($payload);
        return false;
    }

    /**
     * Détecte automatiquement les menaces et les signale
     *
     * @return bool True si une menace a été détectée et signalée
     */
    public function detectAndSignal(): bool
    {
        $detectedPattern = null;

        // Vérification URI complète (chemin + paramètres)
        if (isset($_SERVER['REQUEST_URI'])) {
            $detectedPattern = $this->checkPatterns($_SERVER['REQUEST_URI'], $this->patterns['uri']);
            if ($detectedPattern) {
                return $this->signalHackingAttempt("URI malveillante détectée: " . $detectedPattern);
            }
        }

        // Vérification des paramètres GET individuellement
        if (!empty($_GET)) {
            foreach ($_GET as $key => $value) {
                // Vérifier la clé du paramètre
                $detectedPattern = $this->checkInjectionPatterns($key, $this->patterns['post']);
                if ($detectedPattern) {
                    return $this->signalHackingAttempt("Paramètre GET malveillant détecté dans la clé '$key': " . $detectedPattern);
                }
                
                // Vérifier la valeur du paramètre
                if (is_string($value)) {
                    $detectedPattern = $this->checkInjectionPatterns($value, $this->patterns['post']);
                    if ($detectedPattern) {
                        return $this->signalHackingAttempt("Paramètre GET malveillant détecté dans la valeur de '$key': " . $detectedPattern);
                    }
                } elseif (is_array($value)) {
                    // Vérifier les tableaux de paramètres GET
                    $serializedValue = json_encode($value);
                    $detectedPattern = $this->checkInjectionPatterns($serializedValue, $this->patterns['post']);
                    if ($detectedPattern) {
                        return $this->signalHackingAttempt("Paramètre GET malveillant détecté dans le tableau '$key': " . $detectedPattern);
                    }
                }
            }
        }

        // Vérification POST avec seuil de taille
        if (!empty($_POST)) {
            $postData = json_encode($_POST);
            $postSize = strlen($postData);

            // Vérification du seuil de taille
            if ($postSize > $this->config['post_size_threshold']) {
                return $this->signalHackingAttempt("Données POST dépassent le seuil de taille: {$postSize} bytes");
            }

            // Vérification des patterns malveillants dans POST
            $detectedPattern = $this->checkInjectionPatterns($postData, $this->patterns['post']);
            if ($detectedPattern) {
                return $this->signalHackingAttempt("Données POST malveillantes détectées: " . $detectedPattern);
            }
        }

        // Vérification headers (si activée)
        if ($this->config['headers_detection_enabled']) {
            $userAgent = $_SERVER['HTTP_USER_AGENT'] ?? '';
            $detectedPattern = $this->checkPatterns($userAgent, $this->patterns['headers']);
            if ($detectedPattern) {
                return $this->signalHackingAttempt("User-Agent malveillant détecté: " . $detectedPattern);
            }
        }

        return false;
    }

    /**
     * Traite la file d'attente des signalements en échec
     *
     * @return int Nombre d'éléments traités avec succès
     */
    public function processQueue(): int
    {
        if (!file_exists($this->queueFile)) {
            return 0;
        }

        $content = file_get_contents($this->queueFile);
        if (empty($content)) {
            return 0;
        }

        $queue = json_decode($content, true);
        if (!is_array($queue)) {
            return 0;
        }

        $processed = 0;
        $remaining = [];

        foreach ($queue as $payload) {
            if ($this->sendPayload($payload)) {
                $processed++;
            } else {
                $remaining[] = $payload;
            }
        }

        // Mise à jour de la file d'attente
        if (empty($remaining)) {
            unlink($this->queueFile);
        } else {
            file_put_contents($this->queueFile, json_encode($remaining, JSON_PRETTY_PRINT));
        }

        return $processed;
    }

    /**
     * Construit la configuration finale en fusionnant les sources
     */
    private function buildConfig(array $config): array
    {
        $finalConfig = self::DEFAULT_CONFIG;

        // Variables d'environnement
        if ($envEndpoint = getenv('CYBERSECMODULE_URL')) {
            $finalConfig['endpoint'] = $envEndpoint;
        }

        // Configuration passée en paramètre (priorité maximale)
        $finalConfig = array_merge($finalConfig, $config);

        // Token d'authentification
        $finalConfig['auth_token'] = $config['auth_token'] ?? getenv('CYBERSECMODULE_TOKEN');

        return $finalConfig;
    }

    /**
     * Construit le payload JSON à envoyer
     */
    private function buildPayload(string $pattern, array $additionalData = []): array
    {
        $ip = $this->getClientIp();
        $timestamp = date('c'); // ISO 8601

        $data = [
            'get' => $_GET,
            'post' => $_POST,
            'server' => $_SERVER,
            'cookie' => $_COOKIE
        ];

        // Ajout des données supplémentaires
        if (!empty($additionalData)) {
            $data['additional'] = $additionalData;
        }

        // Sanitisation des données sensibles
        $data = $this->sanitizeData($data);

        return [
            'ip' => $ip,
            'timestamp' => $timestamp,
            'pattern' => $pattern,
            'data' => $data
        ];
    }

    /**
     * Sanitise les données en hachant les éléments sensibles
     */
    private function sanitizeData(array $data): array
    {
        foreach ($data as $section => &$sectionData) {
            if (is_array($sectionData)) {
                foreach ($sectionData as $key => &$value) {
                    if (in_array($key, self::SENSITIVE_KEYS) || $this->isSensitiveKey($key)) {
                        $value = hash('sha256', (string)$value);
                    }
                }
            }
        }

        return $data;
    }

    /**
     * Vérifie si une clé est sensible (cookies de session, etc.)
     */
    private function isSensitiveKey(string $key): bool
    {
        $sensitivePatterns = [
            '/^SESS_/',
            '/^PHPSESSID/',
            '/session/i',
            '/token/i',
            '/auth/i',
            '/password/i',
            '/passwd/i'
        ];

        foreach ($sensitivePatterns as $pattern) {
            if (preg_match($pattern, $key)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Envoie le payload vers l'endpoint
     */
    private function sendPayload(array $payload): bool
    {
        $retries = 0;
        $maxRetries = $this->config['max_retries'];

        while ($retries < $maxRetries) {
            try {
                if ($this->performHttpRequest($payload)) {
                    return true;
                }
            } catch (CyberSecNetworkException $e) {
                // Log l'erreur mais continue les tentatives si possible
                error_log("CyberSecModule Error: " . $e->getMessage());
            } catch (CyberSecConfigurationException $e) {
                // Log l'erreur de configuration
                error_log("CyberSecModule Configuration Error: " . $e->getMessage());
                
                // Pour les erreurs de configuration critiques (comme token manquant),
                // pas la peine de refaire des tentatives, mais ne pas faire planter l'app
                return false;
            } catch (Exception $e) {
                // Catch any other exception
                error_log("CyberSecModule Unexpected Error: " . $e->getMessage());
            }

            $retries++;
            if ($retries < $maxRetries) {
                // Backoff exponentiel: 1s, 2s, 4s
                sleep(pow(2, $retries - 1));
            }
        }

        return false;
    }

    /**
     * Effectue la requête HTTP
     * 
     * @throws CyberSecConfigurationException Si cURL n'est pas disponible
     * @throws CyberSecNetworkException Si une erreur réseau survient
     */
    private function performHttpRequest(array $payload): bool
    {
        if (!function_exists('curl_init')) {
            throw new CyberSecConfigurationException('cURL extension is required');
        }

        // Validation de la clé d'authentification
        if (empty($this->config['auth_token'])) {
            throw new CyberSecConfigurationException('Token d\'authentification requis et ne peut pas être vide');
        }

        $ch = curl_init();
        $jsonData = json_encode($payload);

        // Compression GZIP optionnelle
        if ($this->config['compression'] && function_exists('gzencode')) {
            $jsonData = gzencode($jsonData);
            $headers = [
                'Content-Type: application/json',
                'Content-Encoding: gzip',
                'Authorization: Bearer ' . $this->config['auth_token']
            ];
        } else {
            $headers = [
                'Content-Type: application/json',
                'Authorization: Bearer ' . $this->config['auth_token']
            ];
        }

        curl_setopt_array($ch, [
            CURLOPT_URL => $this->config['endpoint'],
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => $jsonData,
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_TIMEOUT => $this->config['timeout'],
            CURLOPT_FOLLOWLOCATION => false,
            CURLOPT_SSL_VERIFYPEER => true,
            CURLOPT_USERAGENT => 'CyberSecModule/1.0'
        ]);

        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $error = curl_error($ch);
        curl_close($ch);

        if ($response === false || !empty($error)) {
            throw new CyberSecNetworkException("HTTP request failed: " . $error);
        }

        // Code de succès accepté selon le cahier des charges: 200 OK
        return $httpCode === 200;
    }

    /**
     * Ajoute un payload à la file d'attente locale
     */
    private function addToQueue(array $payload): void
    {
        $queue = [];

        if (file_exists($this->queueFile)) {
            $content = file_get_contents($this->queueFile);
            if (!empty($content)) {
                $queue = json_decode($content, true) ?: [];
            }
        }

        $queue[] = $payload;

        // Limitation de la taille de la file (max 100 éléments)
        if (count($queue) > 100) {
            $queue = array_slice($queue, -100);
        }

        file_put_contents($this->queueFile, json_encode($queue, JSON_PRETTY_PRINT));
    }

    /**
     * Vérifie les motifs dans une chaîne
     */
    private function checkPatterns(string $input, array $patterns): ?string
    {
        foreach ($patterns as $pattern) {
            if (stripos($input, $pattern) !== false) {
                return $pattern;
            }
        }

        return null;
    }

    /**
     * Charge les patterns depuis un fichier externe ou utilise les patterns par défaut
     *
     * @return array
     */
    private function loadPatterns(): array
    {
        $patterns = SecurityPatterns::getAllPatterns();

        // Si un fichier de patterns est spécifié, le charger
        if (!empty($this->config['patterns_file'])) {
            try {
                $externalPatterns = $this->loadPatternsFromFile($this->config['patterns_file']);
                if ($externalPatterns !== null) {
                    // Fusionner avec les patterns par défaut
                    foreach ($externalPatterns as $category => $categoryPatterns) {
                        if (isset($patterns[$category]) && is_array($categoryPatterns)) {
                            $patterns[$category] = array_merge($patterns[$category], $categoryPatterns);
                        }
                    }
                }
            } catch (CyberSecFileException | CyberSecConfigurationException $e) {
                // En cas d'erreur avec le fichier de patterns, continuer avec les patterns par défaut
                // et lancer l'exception pour informer l'utilisateur
                throw $e;
            }
        }

        return $patterns;
    }

    /**
     * Charge les patterns depuis un fichier JSON ou YAML
     *
     * @param string $filePath
     * @return array|null
     * @throws CyberSecFileException Si le fichier ne peut pas être lu ou est malformé
     * @throws CyberSecConfigurationException Si l'extension YAML n'est pas disponible ou le format n'est pas supporté
     */
    private function loadPatternsFromFile(string $filePath): ?array
    {
        if (!file_exists($filePath)) {
            throw new CyberSecFileException("Fichier de patterns introuvable: $filePath");
        }

        $content = file_get_contents($filePath);
        if ($content === false) {
            throw new CyberSecFileException("Impossible de lire le fichier de patterns: $filePath");
        }

        $extension = strtolower(pathinfo($filePath, PATHINFO_EXTENSION));

        switch ($extension) {
            case 'json':
                $patterns = json_decode($content, true);
                if (json_last_error() !== JSON_ERROR_NONE) {
                    throw new CyberSecFileException("Erreur JSON dans le fichier de patterns: " . json_last_error_msg());
                }
                return $patterns;

            case 'yml':
            case 'yaml':
                if (function_exists('yaml_parse')) {
                    // @phpstan-ignore-next-line Function yaml_parse is dynamically checked
                    $patterns = \yaml_parse($content);
                    if ($patterns === false) {
                        throw new CyberSecFileException("Erreur YAML dans le fichier de patterns: $filePath");
                    }
                    return $patterns;
                } else {
                    throw new CyberSecConfigurationException("Extension YAML non disponible");
                }

            default:
                throw new CyberSecConfigurationException("Format de fichier non supporté: $extension");
        }
    }

    /**
     * Récupère l'adresse IP du client
     */
    private function getClientIp(): string
    {
        $ipKeys = [
            'HTTP_CF_CONNECTING_IP',     // Cloudflare
            'HTTP_CLIENT_IP',
            'HTTP_X_FORWARDED_FOR',
            'HTTP_X_FORWARDED',
            'HTTP_FORWARDED_FOR',
            'HTTP_FORWARDED',
            'REMOTE_ADDR'
        ];

        foreach ($ipKeys as $key) {
            if (!empty($_SERVER[$key])) {
                $ip = $_SERVER[$key];

                // Si plusieurs IPs (proxy chain), prendre la première
                if (strpos($ip, ',') !== false) {
                    $ip = explode(',', $ip)[0];
                }

                $ip = trim($ip);

                // Validation IP
                if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
                    return $ip;
                }
            }
        }

        return $_SERVER['REMOTE_ADDR'] ?? 'unknown';
    }

    /**
     * Vérifie les injections dans une chaîne avec gestion des regex et patterns simples
     */
    private function checkInjectionPatterns(string $input, array $patterns): ?string
    {
        foreach ($patterns as $pattern) {
            // Pour les patterns qui contiennent \s+ (regex), utiliser preg_match
            if (strpos($pattern, '\\s+') !== false) {
                $escapedPattern = str_replace('/', '\/', $pattern);
                if (preg_match('/' . $escapedPattern . '/i', $input)) {
                    return $pattern;
                }
            } else {
                // Pour les patterns simples, utiliser stripos
                if (stripos($input, $pattern) !== false) {
                    return $pattern;
                }
            }
        }

        return null;
    }
}
